"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var blur_1 = require("./blur");
var offScreenCanvases = {};
function isSafari() {
    return (/^((?!chrome|android).)*safari/i.test(navigator.userAgent));
}
function flipCanvasHorizontal(canvas) {
    var ctx = canvas.getContext('2d');
    ctx.scale(-1, 1);
    ctx.translate(-canvas.width, 0);
}
function compost(ctx, image, compostOperation) {
    ctx.globalCompositeOperation = compostOperation;
    ctx.drawImage(image, 0, 0);
}
function createOffScreenCanvas() {
    var offScreenCanvas = document.createElement('canvas');
    return offScreenCanvas;
}
function ensureOffscreenCanvasCreated(id) {
    if (!offScreenCanvases[id]) {
        offScreenCanvases[id] = createOffScreenCanvas();
    }
    return offScreenCanvases[id];
}
function drawBlurredImageToCanvas(image, bokehBlurAmount, canvas) {
    var height = image.height, width = image.width;
    var ctx = canvas.getContext('2d');
    canvas.width = width;
    canvas.height = height;
    ctx.clearRect(0, 0, width, height);
    ctx.save();
    if (isSafari()) {
        blur_1.cpuBlur(canvas, image, bokehBlurAmount);
    }
    else {
        ctx.filter = "blur(" + bokehBlurAmount + "px)";
        ctx.drawImage(image, 0, 0, width, height);
    }
    ctx.restore();
}
function renderImageToCanvas(image, width, height, canvas) {
    canvas.width = width;
    canvas.height = height;
    var ctx = canvas.getContext('2d');
    ctx.putImageData(image, 0, 0);
}
exports.renderImageToCanvas = renderImageToCanvas;
function toMaskImageData(mask, height, width, invertMask, darknessLevel) {
    if (darknessLevel === void 0) { darknessLevel = 0.7; }
    var bytes = new Uint8ClampedArray(width * height * 4);
    var multiplier = Math.round(255 * darknessLevel);
    for (var i = 0; i < height * width; ++i) {
        var shouldMask = invertMask ? 1 - mask[i] : mask[i];
        var alpha = shouldMask * multiplier;
        var j = i * 4;
        bytes[j + 0] = 0;
        bytes[j + 1] = 0;
        bytes[j + 2] = 0;
        bytes[j + 3] = Math.round(alpha);
    }
    return new ImageData(bytes, width, height);
}
function drawBodyMaskOnCanvas(image, segmentation, canvas, flipHorizontal) {
    if (flipHorizontal === void 0) { flipHorizontal = true; }
    var height = image.height, width = image.width;
    var invertMask = true;
    var maskImage = toMaskImageData(segmentation, height, width, invertMask);
    var maskCanvas = ensureOffscreenCanvasCreated('mask');
    renderImageToCanvas(maskImage, width, height, maskCanvas);
    canvas.width = width;
    canvas.height = height;
    var ctx = canvas.getContext('2d');
    ctx.save();
    if (flipHorizontal) {
        flipCanvasHorizontal(canvas);
    }
    ctx.drawImage(image, 0, 0);
    compost(ctx, maskCanvas, 'source-atop');
    ctx.restore();
}
exports.drawBodyMaskOnCanvas = drawBodyMaskOnCanvas;
function drawBokehEffectOnCanvas(canvas, image, segmentation, bokehBlurAmount, flipHorizontal) {
    if (bokehBlurAmount === void 0) { bokehBlurAmount = 3; }
    if (flipHorizontal === void 0) { flipHorizontal = true; }
    var height = image.height, width = image.width;
    var blurredCanvas = ensureOffscreenCanvasCreated('blur');
    drawBlurredImageToCanvas(image, bokehBlurAmount, blurredCanvas);
    var invertMask = false;
    var darknessLevel = 1.;
    var invertedMaskImage = toMaskImageData(segmentation, height, width, invertMask, darknessLevel);
    var maskWhatsNotThePersonCanvas = ensureOffscreenCanvasCreated('mask');
    renderImageToCanvas(invertedMaskImage, width, height, maskWhatsNotThePersonCanvas);
    var blurredCtx = blurredCanvas.getContext('2d');
    blurredCtx.save();
    compost(blurredCtx, maskWhatsNotThePersonCanvas, 'destination-out');
    blurredCtx.restore();
    var ctx = canvas.getContext('2d');
    ctx.save();
    if (flipHorizontal) {
        flipCanvasHorizontal(canvas);
    }
    ctx.drawImage(image, 0, 0);
    compost(ctx, maskWhatsNotThePersonCanvas, 'destination-in');
    compost(ctx, blurredCanvas, 'source-over');
    ctx.restore();
}
exports.drawBokehEffectOnCanvas = drawBokehEffectOnCanvas;
function toColoredPartImage(partSegmentation, partColors, width, height, alpha) {
    var bytes = new Uint8ClampedArray(width * height * 4);
    for (var i = 0; i < height * width; ++i) {
        var partId = Math.round(partSegmentation[i]);
        var j = i * 4;
        if (partId === -1) {
            bytes[j + 0] = 0;
            bytes[j + 1] = 0;
            bytes[j + 2] = 0;
            bytes[j + 3] = Math.round(255 * alpha);
        }
        else {
            var color = partColors[partId];
            if (!color) {
                throw new Error("No color could be found for part id " + partId);
            }
            bytes[j + 0] = color[0];
            bytes[j + 1] = color[1];
            bytes[j + 2] = color[2];
            bytes[j + 3] = Math.round(255 * alpha);
        }
    }
    return new ImageData(bytes, width, height);
}
function drawBodySegmentsOnCanvas(canvas, input, partSegmentation, partColors, coloredPartImageAlpha, flipHorizontal) {
    if (coloredPartImageAlpha === void 0) { coloredPartImageAlpha = 0.7; }
    if (flipHorizontal === void 0) { flipHorizontal = true; }
    var height = input.height, width = input.width;
    canvas.width = width;
    canvas.height = height;
    var coloredPartImage = toColoredPartImage(partSegmentation, partColors, width, height, coloredPartImageAlpha);
    var partImageCanvas = ensureOffscreenCanvasCreated('partImage');
    renderImageToCanvas(coloredPartImage, width, height, partImageCanvas);
    var ctx = canvas.getContext('2d');
    ctx.save();
    if (flipHorizontal) {
        flipCanvasHorizontal(canvas);
    }
    ctx.drawImage(input, 0, 0);
    compost(ctx, partImageCanvas, 'source-over');
    ctx.restore();
}
exports.drawBodySegmentsOnCanvas = drawBodySegmentsOnCanvas;
//# sourceMappingURL=output_rendering_util.js.map