// @tensorflow/tfjs-models Copyright 2018 Google
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@tensorflow/tfjs')) :
  typeof define === 'function' && define.amd ? define(['exports', '@tensorflow/tfjs'], factory) :
  (factory((global.personSegmentation = {}),global.tf));
}(this, (function (exports,tf) { 'use strict';

  var mobileNet100Architecture = [
      ['conv2d', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1]
  ];
  var mobileNet75Architecture = [
      ['conv2d', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1]
  ];
  var mobileNet50Architecture = [
      ['conv2d', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 2],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1],
      ['separableConv', 1]
  ];
  var mobileNet25Architecture = mobileNet50Architecture;
  var VALID_OUTPUT_STRIDES = [8, 16, 32];
  function assertValidOutputStride(outputStride) {
      tf.util.assert(typeof outputStride === 'number', 'outputStride is not a number');
      tf.util.assert(VALID_OUTPUT_STRIDES.indexOf(outputStride) >= 0, "outputStride of " + outputStride + " is invalid. " +
          "It must be either 8, 16, or 32");
  }
  var mobileNetArchitectures = {
      100: mobileNet100Architecture,
      75: mobileNet75Architecture,
      50: mobileNet50Architecture,
      25: mobileNet25Architecture
  };
  function toOutputStridedLayers(convolutionDefinition, outputStride) {
      var currentStride = 1;
      var rate = 1;
      return convolutionDefinition.map(function (_a, blockId) {
          var convType = _a[0], stride = _a[1];
          var layerStride, layerRate;
          if (currentStride === outputStride) {
              layerStride = 1;
              layerRate = rate;
              rate *= stride;
          }
          else {
              layerStride = stride;
              layerRate = 1;
              currentStride *= stride;
          }
          return {
              blockId: blockId,
              convType: convType,
              stride: layerStride,
              rate: layerRate,
              outputStride: currentStride
          };
      });
  }
  var MobileNet = (function () {
      function MobileNet(modelWeights, convolutionDefinitions) {
          this.PREPROCESS_DIVISOR = tf.scalar(255.0 / 2);
          this.ONE = tf.scalar(1.0);
          this.modelWeights = modelWeights;
          this.convolutionDefinitions = convolutionDefinitions;
      }
      MobileNet.prototype.predict = function (input, outputStride) {
          var _this = this;
          var normalized = tf.div(input.toFloat(), this.PREPROCESS_DIVISOR);
          var preprocessedInput = tf.sub(normalized, this.ONE);
          var layers = toOutputStridedLayers(this.convolutionDefinitions, outputStride);
          return layers.reduce(function (previousLayer, _a) {
              var blockId = _a.blockId, stride = _a.stride, convType = _a.convType, rate = _a.rate;
              if (convType === 'conv2d') {
                  return _this.conv(previousLayer, stride, blockId);
              }
              else if (convType === 'separableConv') {
                  return _this.separableConv(previousLayer, stride, blockId, rate);
              }
              else {
                  throw Error("Unknown conv type of " + convType);
              }
          }, preprocessedInput);
      };
      MobileNet.prototype.convToOutput = function (mobileNetOutput, outputLayerName) {
          return mobileNetOutput.conv2d(this.weights(outputLayerName), 1, 'same')
              .add(this.convBias(outputLayerName, false));
      };
      MobileNet.prototype.conv = function (inputs, stride, blockId) {
          var weights = this.weights("Conv2d_" + String(blockId));
          var a = inputs.conv2d(weights, stride, 'same');
          var b = a.add(this.convBias("Conv2d_" + String(blockId)));
          return b.clipByValue(0, 6);
      };
      MobileNet.prototype.separableConv = function (inputs, stride, blockID, dilations) {
          if (dilations === void 0) { dilations = 1; }
          var dwLayer = "Conv2d_" + String(blockID) + "_depthwise";
          var pwLayer = "Conv2d_" + String(blockID) + "_pointwise";
          var x1 = inputs
              .depthwiseConv2D(this.depthwiseWeights(dwLayer), stride, 'same', 'NHWC', dilations)
              .add(this.depthwiseBias(dwLayer))
              .clipByValue(0, 6);
          var x2 = x1.conv2d(this.weights(pwLayer), [1, 1], 'same')
              .add(this.convBias(pwLayer))
              .clipByValue(0, 6);
          return x2;
      };
      MobileNet.prototype.weights = function (layerName) {
          return this.modelWeights.weights(layerName);
      };
      MobileNet.prototype.convBias = function (layerName, doublePrefix) {
          if (doublePrefix === void 0) { doublePrefix = true; }
          return this.modelWeights.convBias(layerName, doublePrefix);
      };
      MobileNet.prototype.depthwiseBias = function (layerName) {
          return this.modelWeights.depthwiseBias(layerName);
      };
      MobileNet.prototype.depthwiseWeights = function (layerName) {
          return this.modelWeights.depthwiseWeights(layerName);
      };
      MobileNet.prototype.dispose = function () {
          this.modelWeights.dispose();
      };
      return MobileNet;
  }());

  var BASE_URL = 'https://storage.googleapis.com/tfjs-models/savedmodel/';
  var checkpoints = {
      1.0: {
          url: BASE_URL + 'posenet_mobilenet_100_partmap/',
          architecture: mobileNetArchitectures[100]
      },
      0.75: {
          url: BASE_URL + 'posenet_mobilenet_075_partmap/',
          architecture: mobileNetArchitectures[75]
      },
      0.5: {
          url: BASE_URL + 'posenet_mobilenet_050_partmap/',
          architecture: mobileNetArchitectures[50]
      },
      0.25: {
          url: BASE_URL + 'posenet_mobilenet_025_partmap/',
          architecture: mobileNetArchitectures[25]
      }
  };

  function toFlattenedOneHotPartMap(partHeatmapScores) {
      var _a = partHeatmapScores.shape, numParts = _a[2];
      var partMapLocations = partHeatmapScores.argMax(2);
      var partMapFlattened = partMapLocations.reshape([-1]);
      return tf.oneHot(partMapFlattened, numParts);
  }
  function clipByMask2d(image, mask) {
      return image.mul(mask);
  }
  function toMask(segmentScores, threshold) {
      return tf.tidy(function () {
          return segmentScores.greater(tf.scalar(threshold)).toInt();
      });
  }
  function decodePartSegmentation(segmentationMask, partHeatmapScores) {
      var _a = partHeatmapScores.shape, partMapHeight = _a[0], partMapWidth = _a[1], numParts = _a[2];
      return tf.tidy(function () {
          var flattenedMap = toFlattenedOneHotPartMap(partHeatmapScores);
          var partNumbers = tf.range(0, numParts, 1, 'int32').expandDims(1);
          var partMapFlattened = flattenedMap.matMul(partNumbers).toInt();
          var partMap = partMapFlattened.reshape([partMapHeight, partMapWidth]);
          var partMapShiftedUpForClipping = partMap.add(tf.scalar(1, 'int32'));
          return clipByMask2d(partMapShiftedUpForClipping, segmentationMask)
              .sub(tf.scalar(1, 'int32'));
      });
  }

  function cpuBlur(canvas, image, blur) {
      var ctx = canvas.getContext('2d');
      var sum = 0;
      var delta = 5;
      var alphaLeft = 1 / (2 * Math.PI * delta * delta);
      var step = blur < 3 ? 1 : 2;
      for (var y = -blur; y <= blur; y += step) {
          for (var x = -blur; x <= blur; x += step) {
              var weight = alphaLeft * Math.exp(-(x * x + y * y) / (2 * delta * delta));
              sum += weight;
          }
      }
      for (var y = -blur; y <= blur; y += step) {
          for (var x = -blur; x <= blur; x += step) {
              ctx.globalAlpha = alphaLeft *
                  Math.exp(-(x * x + y * y) / (2 * delta * delta)) / sum * blur;
              ctx.drawImage(image, x, y);
          }
      }
      ctx.globalAlpha = 1;
  }

  var offScreenCanvases = {};
  function isSafari() {
      return (/^((?!chrome|android).)*safari/i.test(navigator.userAgent));
  }
  function flipCanvasHorizontal(canvas) {
      var ctx = canvas.getContext('2d');
      ctx.scale(-1, 1);
      ctx.translate(-canvas.width, 0);
  }
  function compost(ctx, image, compostOperation) {
      ctx.globalCompositeOperation = compostOperation;
      ctx.drawImage(image, 0, 0);
  }
  function createOffScreenCanvas() {
      var offScreenCanvas = document.createElement('canvas');
      return offScreenCanvas;
  }
  function ensureOffscreenCanvasCreated(id) {
      if (!offScreenCanvases[id]) {
          offScreenCanvases[id] = createOffScreenCanvas();
      }
      return offScreenCanvases[id];
  }
  function drawBlurredImageToCanvas(image, bokehBlurAmount, canvas) {
      var height = image.height, width = image.width;
      var ctx = canvas.getContext('2d');
      canvas.width = width;
      canvas.height = height;
      ctx.clearRect(0, 0, width, height);
      ctx.save();
      if (isSafari()) {
          cpuBlur(canvas, image, bokehBlurAmount);
      }
      else {
          ctx.filter = "blur(" + bokehBlurAmount + "px)";
          ctx.drawImage(image, 0, 0, width, height);
      }
      ctx.restore();
  }
  function renderImageToCanvas(image, width, height, canvas) {
      canvas.width = width;
      canvas.height = height;
      var ctx = canvas.getContext('2d');
      ctx.putImageData(image, 0, 0);
  }
  function toMaskImageData(mask, height, width, invertMask, darknessLevel) {
      if (darknessLevel === void 0) { darknessLevel = 0.7; }
      var bytes = new Uint8ClampedArray(width * height * 4);
      var multiplier = Math.round(255 * darknessLevel);
      for (var i = 0; i < height * width; ++i) {
          var shouldMask = invertMask ? 1 - mask[i] : mask[i];
          var alpha = shouldMask * multiplier;
          var j = i * 4;
          bytes[j + 0] = 0;
          bytes[j + 1] = 0;
          bytes[j + 2] = 0;
          bytes[j + 3] = Math.round(alpha);
      }
      return new ImageData(bytes, width, height);
  }
  function drawBodyMaskOnCanvas(image, segmentation, canvas, flipHorizontal) {
      if (flipHorizontal === void 0) { flipHorizontal = true; }
      var height = image.height, width = image.width;
      var invertMask = true;
      var maskImage = toMaskImageData(segmentation, height, width, invertMask);
      var maskCanvas = ensureOffscreenCanvasCreated('mask');
      renderImageToCanvas(maskImage, width, height, maskCanvas);
      canvas.width = width;
      canvas.height = height;
      var ctx = canvas.getContext('2d');
      ctx.save();
      if (flipHorizontal) {
          flipCanvasHorizontal(canvas);
      }
      ctx.drawImage(image, 0, 0);
      compost(ctx, maskCanvas, 'source-atop');
      ctx.restore();
  }
  function drawBokehEffectOnCanvas(canvas, image, segmentation, bokehBlurAmount, flipHorizontal) {
      if (bokehBlurAmount === void 0) { bokehBlurAmount = 3; }
      if (flipHorizontal === void 0) { flipHorizontal = true; }
      var height = image.height, width = image.width;
      var blurredCanvas = ensureOffscreenCanvasCreated('blur');
      drawBlurredImageToCanvas(image, bokehBlurAmount, blurredCanvas);
      var invertMask = false;
      var darknessLevel = 1.;
      var invertedMaskImage = toMaskImageData(segmentation, height, width, invertMask, darknessLevel);
      var maskWhatsNotThePersonCanvas = ensureOffscreenCanvasCreated('mask');
      renderImageToCanvas(invertedMaskImage, width, height, maskWhatsNotThePersonCanvas);
      var blurredCtx = blurredCanvas.getContext('2d');
      blurredCtx.save();
      compost(blurredCtx, maskWhatsNotThePersonCanvas, 'destination-out');
      blurredCtx.restore();
      var ctx = canvas.getContext('2d');
      ctx.save();
      if (flipHorizontal) {
          flipCanvasHorizontal(canvas);
      }
      ctx.drawImage(image, 0, 0);
      compost(ctx, maskWhatsNotThePersonCanvas, 'destination-in');
      compost(ctx, blurredCanvas, 'source-over');
      ctx.restore();
  }
  function toColoredPartImage(partSegmentation, partColors, width, height, alpha) {
      var bytes = new Uint8ClampedArray(width * height * 4);
      for (var i = 0; i < height * width; ++i) {
          var partId = Math.round(partSegmentation[i]);
          var j = i * 4;
          if (partId === -1) {
              bytes[j + 0] = 0;
              bytes[j + 1] = 0;
              bytes[j + 2] = 0;
              bytes[j + 3] = Math.round(255 * alpha);
          }
          else {
              var color = partColors[partId];
              if (!color) {
                  throw new Error("No color could be found for part id " + partId);
              }
              bytes[j + 0] = color[0];
              bytes[j + 1] = color[1];
              bytes[j + 2] = color[2];
              bytes[j + 3] = Math.round(255 * alpha);
          }
      }
      return new ImageData(bytes, width, height);
  }
  function drawBodySegmentsOnCanvas(canvas, input, partSegmentation, partColors, coloredPartImageAlpha, flipHorizontal) {
      if (coloredPartImageAlpha === void 0) { coloredPartImageAlpha = 0.7; }
      if (flipHorizontal === void 0) { flipHorizontal = true; }
      var height = input.height, width = input.width;
      canvas.width = width;
      canvas.height = height;
      var coloredPartImage = toColoredPartImage(partSegmentation, partColors, width, height, coloredPartImageAlpha);
      var partImageCanvas = ensureOffscreenCanvasCreated('partImage');
      renderImageToCanvas(coloredPartImage, width, height, partImageCanvas);
      var ctx = canvas.getContext('2d');
      ctx.save();
      if (flipHorizontal) {
          flipCanvasHorizontal(canvas);
      }
      ctx.drawImage(input, 0, 0);
      compost(ctx, partImageCanvas, 'source-over');
      ctx.restore();
  }

  var partChannels = [
      'left_face',
      'right_face',
      'right_upper_leg_front',
      'right_lower_leg_back',
      'right_upper_leg_back',
      'left_lower_leg_front',
      'left_upper_leg_front',
      'left_upper_leg_back',
      'left_lower_leg_back',
      'right_feet',
      'right_lower_leg_front',
      'left_feet',
      'torso_front',
      'torso_back',
      'right_upper_arm_front',
      'right_upper_arm_back',
      'right_lower_arm_back',
      'left_lower_arm_front',
      'left_upper_arm_front',
      'left_upper_arm_back',
      'left_lower_arm_back',
      'right_hand',
      'right_lower_arm_front',
      'left_hand'
  ];

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  function __awaiter(thisArg, _arguments, P, generator) {
      return new (P || (P = Promise))(function (resolve, reject) {
          function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
          function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
          function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
          step((generator = generator.apply(thisArg, _arguments || [])).next());
      });
  }

  function __generator(thisArg, body) {
      var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
      return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
      function verb(n) { return function (v) { return step([n, v]); }; }
      function step(op) {
          if (f) throw new TypeError("Generator is already executing.");
          while (_) try {
              if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
              if (y = 0, t) op = [op[0] & 2, t.value];
              switch (op[0]) {
                  case 0: case 1: t = op; break;
                  case 4: _.label++; return { value: op[1], done: false };
                  case 5: _.label++; y = op[1]; op = [0]; continue;
                  case 7: op = _.ops.pop(); _.trys.pop(); continue;
                  default:
                      if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                      if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                      if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                      if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                      if (t[2]) _.ops.pop();
                      _.trys.pop(); continue;
              }
              op = body.call(thisArg, _);
          } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
          if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
      }
  }

  var ModelWeights = (function () {
      function ModelWeights(frozenModel) {
          this.frozenModel = frozenModel;
      }
      ModelWeights.prototype.weights = function (layerName) {
          return this.getVariable("MobilenetV1/" + layerName + "/weights");
      };
      ModelWeights.prototype.convBias = function (layerName, doublePrefix) {
          if (doublePrefix === void 0) { doublePrefix = true; }
          return this.getVariable("MobilenetV1/" + layerName + "/Conv2D_bias");
      };
      ModelWeights.prototype.depthwiseBias = function (layerName) {
          return this.getVariable("MobilenetV1/" + layerName + "/depthwise_bias");
      };
      ModelWeights.prototype.depthwiseWeights = function (layerName) {
          return this.getVariable("MobilenetV1/" + layerName + "/depthwise_weights");
      };
      ModelWeights.prototype.getVariable = function (name) {
          return this.frozenModel.weights["" + name][0];
      };
      ModelWeights.prototype.dispose = function () {
          this.frozenModel.dispose();
      };
      return ModelWeights;
  }());

  function getInputTensorDimensions(input) {
      return input instanceof tf.Tensor ? [input.shape[0], input.shape[1]] :
          [input.height, input.width];
  }
  function toInputTensor(input) {
      return input instanceof tf.Tensor ? input : tf.fromPixels(input);
  }
  function resizeAndPadTo(input, _a, flipHorizontal) {
      var targetH = _a[0], targetW = _a[1];
      if (flipHorizontal === void 0) { flipHorizontal = false; }
      var _b = getInputTensorDimensions(input), height = _b[0], width = _b[1];
      var targetAspect = targetW / targetH;
      var aspect = width / height;
      var resizeW;
      var resizeH;
      var padL;
      var padR;
      var padT;
      var padB;
      if (aspect > targetAspect) {
          resizeW = targetW;
          resizeH = Math.ceil(resizeW / aspect);
          var padHeight = targetH - resizeH;
          padL = 0;
          padR = 0;
          padT = Math.floor(padHeight / 2);
          padB = targetH - (resizeH + padT);
      }
      else {
          resizeH = targetH;
          resizeW = Math.ceil(targetH / aspect);
          var padWidth = targetW - resizeW;
          padL = Math.floor(padWidth / 2);
          padR = targetW - (resizeW + padL);
          padT = 0;
          padB = 0;
      }
      var resizedAndPadded = tf.tidy(function () {
          var imageTensor = toInputTensor(input);
          var resized;
          if (flipHorizontal) {
              resized = imageTensor.reverse(1).resizeBilinear([resizeH, resizeW]);
          }
          else {
              resized = imageTensor.resizeBilinear([resizeH, resizeW]);
          }
          var padded = tf.pad3d(resized, [[padT, padB], [padL, padR], [0, 0]]);
          return padded;
      });
      return { resizedAndPadded: resizedAndPadded, paddedBy: [[padT, padB], [padL, padR]] };
  }
  function scaleAndCropToInputTensorShape(tensor, _a, _b, _c) {
      var inputTensorHeight = _a[0], inputTensorWidth = _a[1];
      var resizedAndPaddedHeight = _b[0], resizedAndPaddedWidth = _b[1];
      var _d = _c[0], padT = _d[0], padB = _d[1], _e = _c[1], padL = _e[0], padR = _e[1];
      return tf.tidy(function () {
          var inResizedAndPaddedSize = tensor.resizeBilinear([resizedAndPaddedHeight, resizedAndPaddedWidth], true);
          return removePaddingAndResizeBack(inResizedAndPaddedSize, [inputTensorHeight, inputTensorWidth], [[padT, padB], [padL, padR]]);
      });
  }
  function removePaddingAndResizeBack(resizedAndPadded, _a, _b) {
      var originalHeight = _a[0], originalWidth = _a[1];
      var _c = _b[0], padT = _c[0], padB = _c[1], _d = _b[1], padL = _d[0], padR = _d[1];
      var _e = resizedAndPadded.shape, height = _e[0], width = _e[1];
      var cropH = height - (padT + padB);
      var cropW = width - (padL + padR);
      return tf.tidy(function () {
          var withPaddingRemoved = tf.slice3d(resizedAndPadded, [padT, padL, 0], [cropH, cropW, resizedAndPadded.shape[2]]);
          var atOriginalSize = withPaddingRemoved.resizeBilinear([originalHeight, originalWidth], true);
          return atOriginalSize;
      });
  }

  var _this = undefined;
  var segmentationModelImageDimensions = [353, 257];
  var PersonSegmentation = (function () {
      function PersonSegmentation(mobileNet) {
          this.mobileNet = mobileNet;
      }
      PersonSegmentation.prototype.predictForSegmentation = function (input, outputStride) {
          var _this = this;
          if (outputStride === void 0) { outputStride = 16; }
          assertValidOutputStride(outputStride);
          return tf.tidy(function () {
              var mobileNetOutput = _this.mobileNet.predict(input, outputStride);
              return _this.mobileNet.convToOutput(mobileNetOutput, 'segment_2')
                  .sigmoid();
          });
      };
      PersonSegmentation.prototype.predictForPartMap = function (input, outputStride) {
          var _this = this;
          if (outputStride === void 0) { outputStride = 16; }
          assertValidOutputStride(outputStride);
          return tf.tidy(function () {
              var mobileNetOutput = _this.mobileNet.predict(input, outputStride);
              var segments = _this.mobileNet.convToOutput(mobileNetOutput, 'segment_2');
              var partHeatmaps = _this.mobileNet.convToOutput(mobileNetOutput, 'part_heatmap_2');
              return {
                  segmentScores: segments.sigmoid(),
                  partHeatmapScores: partHeatmaps.sigmoid()
              };
          });
      };
      PersonSegmentation.prototype.estimatePersonSegmentation = function (input, flipHorizontal, outputStride, segmentationThreshold) {
          if (flipHorizontal === void 0) { flipHorizontal = false; }
          if (outputStride === void 0) { outputStride = 16; }
          if (segmentationThreshold === void 0) { segmentationThreshold = 0.5; }
          return __awaiter(this, void 0, void 0, function () {
              var _a, height, width, segmentation, result;
              var _this = this;
              return __generator(this, function (_b) {
                  switch (_b.label) {
                      case 0:
                          assertValidOutputStride(outputStride);
                          _a = getInputTensorDimensions(input), height = _a[0], width = _a[1];
                          segmentation = tf.tidy(function () {
                              var _a = resizeAndPadTo(input, segmentationModelImageDimensions, flipHorizontal), resizedAndPadded = _a.resizedAndPadded, paddedBy = _a.paddedBy;
                              var segmentScores = _this.predictForSegmentation(resizedAndPadded, outputStride);
                              var _b = resizedAndPadded.shape, resizedHeight = _b[0], resizedWidth = _b[1];
                              var scaledSegmentScores = scaleAndCropToInputTensorShape(segmentScores, [height, width], [resizedHeight, resizedWidth], paddedBy);
                              var mask = toMask(scaledSegmentScores.squeeze(), segmentationThreshold);
                              if (flipHorizontal) {
                                  return mask.reverse(1);
                              }
                              else {
                                  return mask;
                              }
                          });
                          return [4, segmentation.data()];
                      case 1:
                          result = _b.sent();
                          segmentation.dispose();
                          return [2, result];
                  }
              });
          });
      };
      PersonSegmentation.prototype.estimatePartSegmentation = function (input, flipHorizontal, outputStride, segmentationThreshold) {
          if (flipHorizontal === void 0) { flipHorizontal = false; }
          if (outputStride === void 0) { outputStride = 16; }
          if (segmentationThreshold === void 0) { segmentationThreshold = 0.5; }
          return __awaiter(this, void 0, void 0, function () {
              var _a, height, width, partSegmentation, result;
              var _this = this;
              return __generator(this, function (_b) {
                  switch (_b.label) {
                      case 0:
                          assertValidOutputStride(outputStride);
                          _a = getInputTensorDimensions(input), height = _a[0], width = _a[1];
                          partSegmentation = tf.tidy(function () {
                              var _a = resizeAndPadTo(input, segmentationModelImageDimensions, flipHorizontal), resizedAndPadded = _a.resizedAndPadded, paddedBy = _a.paddedBy;
                              var _b = _this.predictForPartMap(resizedAndPadded, outputStride), segmentScores = _b.segmentScores, partHeatmapScores = _b.partHeatmapScores;
                              var _c = resizedAndPadded.shape, resizedHeight = _c[0], resizedWidth = _c[1];
                              var scaledSegmentScores = scaleAndCropToInputTensorShape(segmentScores, [height, width], [resizedHeight, resizedWidth], paddedBy);
                              var scaledPartHeatmapScore = scaleAndCropToInputTensorShape(partHeatmapScores, [height, width], [resizedHeight, resizedWidth], paddedBy);
                              var segmentationMask = toMask(scaledSegmentScores.squeeze(), segmentationThreshold);
                              var partSegmentation = decodePartSegmentation(segmentationMask, scaledPartHeatmapScore);
                              if (flipHorizontal) {
                                  return partSegmentation.reverse(1);
                              }
                              else {
                                  return partSegmentation;
                              }
                          });
                          return [4, partSegmentation.data()];
                      case 1:
                          result = _b.sent();
                          partSegmentation.dispose();
                          return [2, result];
                  }
              });
          });
      };
      PersonSegmentation.prototype.dispose = function () {
          this.mobileNet.dispose();
      };
      return PersonSegmentation;
  }());
  function load(multiplier) {
      if (multiplier === void 0) { multiplier = 0.75; }
      return __awaiter(this, void 0, void 0, function () {
          var possibleMultipliers, mobileNet;
          return __generator(this, function (_a) {
              switch (_a.label) {
                  case 0:
                      if (tf == null) {
                          throw new Error("Cannot find TensorFlow.js. If you are using a <script> tag, please " +
                              "also include @tensorflow/tfjs on the page before using this model.");
                      }
                      possibleMultipliers = Object.keys(checkpoints);
                      tf.util.assert(typeof multiplier === 'number', "got multiplier type of " + typeof multiplier + " when it should be a " +
                          "number.");
                      tf.util.assert(possibleMultipliers.indexOf(multiplier.toString()) >= 0, "invalid multiplier value of " + multiplier +
                          ".  No checkpoint exists for that " +
                          ("multiplier. Must be one of " + possibleMultipliers.join(',') + "."));
                      return [4, mobilenetLoader.load(multiplier)];
                  case 1:
                      mobileNet = _a.sent();
                      return [2, new PersonSegmentation(mobileNet)];
              }
          });
      });
  }
  var mobilenetLoader = {
      load: function (multiplier) { return __awaiter(_this, void 0, void 0, function () {
          var checkpoint, baseUrl, model, weights;
          return __generator(this, function (_a) {
              switch (_a.label) {
                  case 0:
                      checkpoint = checkpoints[multiplier];
                      baseUrl = checkpoint.url;
                      return [4, tf.loadFrozenModel(baseUrl + "tensorflowjs_model.pb", baseUrl + "weights_manifest.json")];
                  case 1:
                      model = _a.sent();
                      weights = new ModelWeights(model);
                      return [2, new MobileNet(weights, checkpoint.architecture)];
              }
          });
      }); }
  };

  exports.checkpoints = checkpoints;
  exports.decodePartSegmentation = decodePartSegmentation;
  exports.toMask = toMask;
  exports.drawBodyMaskOnCanvas = drawBodyMaskOnCanvas;
  exports.drawBodySegmentsOnCanvas = drawBodySegmentsOnCanvas;
  exports.drawBokehEffectOnCanvas = drawBokehEffectOnCanvas;
  exports.partChannels = partChannels;
  exports.load = load;
  exports.PersonSegmentation = PersonSegmentation;
  exports.resizeAndPadTo = resizeAndPadTo;
  exports.scaleAndCropToInputTensorShape = scaleAndCropToInputTensorShape;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
